=== modified file 'launcher/SwitcherController.cpp'
--- launcher/SwitcherController.cpp	2015-04-03 15:46:16 +0000
+++ launcher/SwitcherController.cpp	2015-10-30 17:16:12 +0000
@@ -45,25 +45,6 @@
 const std::string VIEW_CONSTRUCT_IDLE = "view-construct-idle";
 const unsigned FADE_DURATION = 80;
 const int XY_OFFSET = 100;
-
-/**
- * Helper comparison functor for sorting application icons.
- */
-bool CompareSwitcherItemsPriority(AbstractLauncherIcon::Ptr const& first,
-                                  AbstractLauncherIcon::Ptr const& second)
-{
-  if (first->GetIconType() == second->GetIconType())
-    return first->SwitcherPriority() > second->SwitcherPriority();
-
-  if (first->GetIconType() == AbstractLauncherIcon::IconType::DESKTOP)
-    return true;
-
-  if (second->GetIconType() == AbstractLauncherIcon::IconType::DESKTOP)
-    return false;
-
-  return first->GetIconType() < second->GetIconType();
-}
-
 }
 
 namespace switcher
@@ -98,7 +79,7 @@
 
 void Controller::Show(ShowMode show,
                       SortMode sort,
-                      std::vector<AbstractLauncherIcon::Ptr> results)
+                      std::vector<AbstractLauncherIcon::Ptr> const& results)
 {
   auto uscreen = UScreen::GetDefault();
   monitor_     = uscreen->GetMonitorWithMouse();
@@ -106,6 +87,16 @@
   impl_->Show(show, sort, results);
 }
 
+void Controller::AddIcon(AbstractLauncherIcon::Ptr const& icon)
+{
+  impl_->AddIcon(icon);
+}
+
+void Controller::RemoveIcon(AbstractLauncherIcon::Ptr const& icon)
+{
+  impl_->RemoveIcon(icon);
+}
+
 void Controller::Select(int index)
 {
   if (Visible())
@@ -290,23 +281,33 @@
     view_->background_color = new_color;
 }
 
-
-void Controller::Impl::Show(ShowMode show, SortMode sort, std::vector<AbstractLauncherIcon::Ptr> results)
+void Controller::Impl::AddIcon(AbstractLauncherIcon::Ptr const& icon)
+{
+  if (!obj_->visible_ || !model_)
+    return;
+
+  model_->AddIcon(icon);
+}
+
+void Controller::Impl::RemoveIcon(AbstractLauncherIcon::Ptr const& icon)
+{
+  if (!obj_->visible_ || !model_)
+    return;
+
+  model_->RemoveIcon(icon);
+}
+
+void Controller::Impl::Show(ShowMode show_mode, SortMode sort_mode, std::vector<AbstractLauncherIcon::Ptr> const& results)
 {
   if (results.empty() || obj_->visible_)
     return;
 
-  if (sort == SortMode::FOCUS_ORDER)
-  {
-    std::sort(results.begin(), results.end(), CompareSwitcherItemsPriority);
-  }
-
-  model_ = std::make_shared<SwitcherModel>(results);
-  obj_->AddChild(model_.get());
+  model_ = std::make_shared<SwitcherModel>(results, (sort_mode == SortMode::FOCUS_ORDER));
+  model_->only_apps_on_viewport = (show_mode == ShowMode::CURRENT_VIEWPORT);
   model_->selection_changed.connect(sigc::mem_fun(this, &Controller::Impl::OnModelSelectionChanged));
   model_->detail_selection.changed.connect([this] (bool) { sources_.Remove(DETAIL_TIMEOUT); });
   model_->request_detail_hide.connect(sigc::mem_fun(this, &Controller::Impl::DetailHide));
-  model_->only_detail_on_viewport = (show == ShowMode::CURRENT_VIEWPORT);
+  obj_->AddChild(model_.get());
 
   SelectFirstItem();
 
@@ -715,7 +716,7 @@
   {
     Window xid = window->window_id();
 
-    if (model_->only_detail_on_viewport && !wm.IsWindowOnCurrentDesktop(xid))
+    if (model_->only_apps_on_viewport && !wm.IsWindowOnCurrentDesktop(xid))
       continue;
 
     uint64_t num = wm.GetWindowActiveNumber(xid);

=== modified file 'launcher/SwitcherController.h'
--- launcher/SwitcherController.h	2015-04-03 15:32:24 +0000
+++ launcher/SwitcherController.h	2015-10-30 17:16:12 +0000
@@ -72,7 +72,7 @@
 };
 
 
-class Controller : public debug::Introspectable
+class Controller : public debug::Introspectable, public sigc::trackable
 {
 public:
   class Impl;
@@ -87,10 +87,12 @@
 
   void Show(ShowMode show,
             SortMode sort,
-            std::vector<launcher::AbstractLauncherIcon::Ptr> results);
+            std::vector<launcher::AbstractLauncherIcon::Ptr> const& results);
   void Hide(bool accept_state=true);
 
-  bool CanShowSwitcher(const std::vector<launcher::AbstractLauncherIcon::Ptr>& resutls) const;
+  bool CanShowSwitcher(std::vector<launcher::AbstractLauncherIcon::Ptr> const& resutls) const;
+  void AddIcon(launcher::AbstractLauncherIcon::Ptr const&);
+  void RemoveIcon(launcher::AbstractLauncherIcon::Ptr const&);
 
   bool Visible();
   nux::Geometry GetInputWindowGeometry() const;

=== modified file 'launcher/SwitcherControllerImpl.h'
--- launcher/SwitcherControllerImpl.h	2014-03-25 19:07:36 +0000
+++ launcher/SwitcherControllerImpl.h	2015-10-30 17:16:12 +0000
@@ -46,10 +46,13 @@
        Controller::WindowCreator const& create_window);
   virtual ~Impl() {}
 
-  void Show(ShowMode show, SortMode sort, std::vector<launcher::AbstractLauncherIcon::Ptr> results);
+  void Show(ShowMode show, SortMode sort, std::vector<launcher::AbstractLauncherIcon::Ptr> const& results);
   void Hide(bool accept_state);
   void DetailHide();
 
+  void AddIcon(launcher::AbstractLauncherIcon::Ptr const&);
+  void RemoveIcon(launcher::AbstractLauncherIcon::Ptr const&);
+
   void StartDetailMode();
   void StopDetailMode();
 

=== modified file 'launcher/SwitcherModel.cpp'
--- launcher/SwitcherModel.cpp	2015-10-30 17:08:02 +0000
+++ launcher/SwitcherModel.cpp	2015-10-30 17:16:12 +0000
@@ -83,7 +83,7 @@
 
 void SwitcherModel::AddIcon(launcher::AbstractLauncherIcon::Ptr const& icon)
 {
-  if (!icon)
+  if (!icon || !icon->ShowInSwitcher(only_apps_on_viewport))
     return;
 
   if (icon->GetQuirk(AbstractLauncherIcon::Quirk::ACTIVE))

